function F = reactive_distDAE(t,Y)
% This file is a modified version of Sigurds flash.m file  

% I. Data:
Ntrays = 20; %The number of trays
NC = 6;      %The number of compounds
R = 8.314472*10^(-3);       %Gas constant [kJ/K*mol]
Mw = [32.0 879.3844 616.95 354.52 92.094 294.47]; %Molecular weights [kg/kmol]
Ntrays_react = 10;           %Number of reactive trays
q = 1.0280;                 %Feed liquid fraction
F = [300 50 0 0 0 0];       %Feed [kmol/h]
F0 = sum(F);                %Total feed [kmol/h]
x_in = F/F0;                %Molar fractions
Vb = 430;                   %Vapour boilup
Dc = 1.8564;                        %Diameter of column [m]
lw = 1.4295;                        %Length of weir [m]
hw = 0.20;                  %Heigth of weir [m]
D = 105;                 %Distillate [kmol/h]
B = 244.9;                 %Bottoms, [kmol/h]
Ref = 3*D;                  %Reflux ratio

% Parameters for the vapour pressure equations:
A_met = 82.718; B_met = -6.9045*10^3; C_met = -8.8622; D_met = 7.4664*10^-6; E_met = 2;
A_TG = 234.71; B_TG = -3.4699*10^4; C_TG = -27.25; D_TG = 1.5475*10^-18; E_TG = 6;
A_DG = -15.931; B_DG = -2111.0; C_DG = 2.4303; D_DG = 8.0567*10^-21; E_DG = 6;
A_MG = 118.95; B_MG = -2.0181*10^4; C_MG = -14.32; D_MG = 9.148*10^-19; E_MG = 6;
A_GL = 99.986; B_GL = -1.3808*10^4; C_GL = -10.088; D_GL = 3.5712*10^-19; E_GL = 6;
A_BD = 105.47; B_BD = -1.4531*10^4; C_BD = -10.986; D_BD = 2.5735*10^-18; E_BD = 6;

% II. Extract present value of states
x = Y(1:(Ntrays+2)*(NC-1),1);   % Compositions of compounds 1-5 on each tray
H = Y(111:132,1);               % Molar holdup on each tray [kmol]  
T = Y(133:154,1);               % Temperature on each tray [K] 
xL = reshape(x,Ntrays+2,NC-1);  % Reshaping the  compositions
HL = reshape(H,Ntrays+2,1);     % Reshaping the molar holdups
TL = reshape(T,Ntrays+2,1);     % Reshaping the temperature

% IIIa. Intermediate calculations
%Liquid and vapour molar flows:
V = ones(Ntrays+1,1);  %V is a vector of the molar vapor flows in the column
V(1,1) = D + Ref;      %From a molar balance over the top of the column
L = ones(Ntrays+1,1);  %L is a vector of the different trays in the column.
L(1,1) = Ref;          %The first liquid molar flow is equal to the reflux.

XX = zeros(Ntrays+2,NC);  x_BD = zeros(Ntrays+2,1);
for i = 1:Ntrays+2
    x_BD(i,1) = 1 - sum(xL(i,:));
    XX(i,:) = [xL(i,:) x_BD(i,1)];
end

%% Calculation of density
% Initialising sizes
rho_MetOH = ones(Ntrays+2,1); rho_TG = ones(Ntrays+2,1); rho_GL = ones(Ntrays+2,1);
rho_BD = ones(Ntrays+2,1); rho_DG = ones(Ntrays+2,1); rho_MG = ones(Ntrays+2,1);
rho = [rho_MetOH rho_TG rho_DG rho_MG rho_GL rho_BD];
rho_l = ones(Ntrays+2,1); rho_L = ones(Ntrays+2,1); x_wt = zeros(Ntrays+2,NC);

% The constants
Ar_met = 2.3267; Br_met = 0.27073; Cr_met = 512.5; Dr_met = 0.24713;
Ar_TG = 0.026085; Br_TG = 0.14259; Cr_TG = 934.6; Dr_TG = 0.28571;
Ar_GL = 0.92382; Br_GL = 0.24386; Cr_GL = 850; Dr_GL = 0.22114;
Ar_BD = 0.20469; Br_BD = 0.23737; Cr_BD = 767.4; Dr_BD = 0.28571;
 
for i = 1:Ntrays+2
    %Density [kmol/m3]
      rho_MetOH(i,1) = Ar_met/(Br_met^(1+(1-(T(i,1)/Cr_met))^Dr_met));
      rho_TG(i,1) = Ar_TG/(Br_TG^(1+(1-(T(i,1)/Cr_TG))^Dr_TG));
      rho_GL(i,1) = Ar_GL/(Br_GL^(1+(1-(T(i,1)/Cr_GL))^Dr_GL));
      rho_BD(i,1) = Ar_BD/(Br_BD^(1+(1-(T(i,1)/Cr_BD))^Dr_BD));
      rho_DG(i,1) = ((2/3)*rho_TG(i,1)) + ((1/3)*rho_BD(i,1)); 
      rho_MG(i,1) = ((1/3)*rho_TG(i,1)) + ((2/3)*rho_BD(i,1)); 
      rho(i,:) = [rho_MetOH(i,1) rho_TG(i,1) rho_DG(i,1) rho_MG(i,1) rho_GL(i,1) rho_BD(i,1)];
      %A common liquid density for the tray [kmol/m3]
      rho_l(i,1) = sum(rho(i,:).*XX(i,:));
      %Weight fraction
      x_wt(i,:) = (XX(i,:)*HL(i,1).*Mw)/sum(XX(i,:)*HL(i,1).*Mw); 
      %A common liquid density for the tray [kg/m3]
      rho_L(i,1) = sum(rho(i,:).*Mw.*x_wt(i,:)); 
end


% Applying Francis weir for the liquid flowrates
% Initializing sizes
V = ones(Ntrays+1,1);  %V is a vector of the molar vapor flows in the column
L = ones(Ntrays+1,1);  %L is a vector of the different trays in the column.

% Applying Francis weir for the liquid flowrates
how = ones(Ntrays,1); Lw = ones(Ntrays,1);
for i = 1:Ntrays
      how(i,1) = (HL(i+1,1)-(pi*hw*rho_l(i+1,1)*0.88*(Dc/2)^2))/(pi*0.88*(Dc/2)^2); %Calculation of how
      Lw(i+1,1) = ((how(i,1)*10^3)/750)^1.5*rho_L(i+1,1)*lw;    %Francis weir [kg/s]
      L(i+1,1) = (Lw(i+1,1)*3600)/sum(XX(i+1,:).*Mw);           %Liquid flow [kmol/h]
end

% Vapour flowrates
V(Ntrays+1,1) = Vb;  %The vapour from the last tray is equal to the vapour boilup

% Assuming constant molar overflow for vapour flowrates
for i = 2:Ntrays
    V(i,1) = V(Ntrays+1,1);
end

% Correction for the feed:
V(1,1) = V(2,1) + ((1-q)*F0);% The vapour flowrate over the feed has been reduced by qF0

% Molar balances over the top and the bottom: 
L(1,1) = Ref;                % The first liquid molar flow is equal to the reflux.

% VLE
y = zeros(Ntrays+2,5);
P = ones(Ntrays+2,1);
for j = 1:Ntrays+2
P_met = 9.869*(10^-6)*exp(A_met + (B_met/TL(j,1)) + (C_met*log(TL(j,1))) + (D_met*TL(j,1)^E_met));
P_TG = 9.869*(10^-6)*exp(A_TG + (B_TG/TL(j,1)) + (C_TG*log(TL(j,1))) + (D_TG*TL(j,1)^E_TG));
P_DG = 9.869*exp(A_DG + (B_DG/TL(j,1)) + (C_DG*log(TL(j,1))) + (D_DG*TL(j,1)^E_DG));
P_MG = 9.869*exp(A_MG + (B_MG/TL(j,1)) + (C_MG*log(TL(j,1))) + (D_MG*TL(j,1)^E_MG));
P_GL = 9.869*(10^-6)*exp(A_GL + (B_GL/TL(j,1)) + (C_GL*log(TL(j,1))) + (D_GL*TL(j,1)^E_GL));
P_BD = 9.869*(10^-6)*exp(A_BD + (B_BD/TL(j,1)) + (C_BD*log(TL(j,1))) + (D_BD*TL(j,1)^E_BD));
x_BD = 1 - sum(xL(j,:));
P(j,1) = (xL(j,1)*P_met)+(xL(j,2)*P_TG)+(xL(j,3)*P_DG)+(xL(j,4)*P_MG)+(xL(j,5)*P_GL)+(x_BD*P_BD);
y(j,1) = (P_met*xL(j,1))/P(j,1);
y(j,2) = (P_TG*xL(j,2))/P(j,1);
y(j,3) = (P_DG*xL(j,3))/P(j,1);
y(j,4) = (P_MG*xL(j,4))/P(j,1);
y(j,5) = (P_GL*xL(j,5))/P(j,1);
end

%% Kinetics
%The sizes of the kinetic constants:
k1 = ones(Ntrays+2,1); k2 = ones(Ntrays+2,1); k3 = ones(Ntrays+2,1);
k4 = ones(Ntrays+2,1); k5 = ones(Ntrays+2,1); k6 = ones(Ntrays+2,1);

for i = 1:Ntrays+2
%Kinetic data for the reaction
%rate constants [m3/kmol*h]
k1(i) = 3.9*3600*(10^7)*exp(-54.9987/(R*TL(i,1)));    
k2(i) = 5.78*3600*(10^5)*exp(-41.5555/(R*TL(i,1)));
k3(i) = 5.906*3600*(10^12)*exp(-83.0942/(R*TL(i,1)));
k4(i) = 9.888*3600*(10^9)*exp(-61.2496/(R*TL(i,1)));
k5(i) = 5.335*3600*(10^3)*exp(-26.8655/(R*TL(i,1)));
k6(i) = 2.1*3600*(10^4)*exp(-40.1162/(R*TL(i,1)));
end

%We need to convert the molar input to concentrations in order to calculate
%the reaction outputs:
C = zeros(Ntrays_react,6); %Creates a matrix with reactive trays as rows and a column for each compound
R = zeros(Ntrays_react,6);
M = zeros(size(C));
Msum = zeros(Ntrays_react,1);
for i = 1:Ntrays_react  
   x_BD = 1 - sum(xL(i+1,:));   %Finding the fraction of biodiesel from mass balance
   XXL = [xL(i+1,:) x_BD];      %The liquid fraction vector includig the 6th state
   vH = HL(i+1,1)*XXL;          %molar holdup vector
   vr = sum(vH./rho(i+1));      %volume holdup
   C(i,:) = vH/vr;              %concentration of each component
%The reactions. R is the gain/loss in moles/h
R_TG = -(k1(i+1)*C(i,2)*C(i,1)) + (k2(i+1)*C(i,3)*C(i,6));
R_DG = (k1(i+1)*C(i,2)*C(i,1)) - (k2(i+1)*C(i,3)*C(i,6)) - (k3(i+1)*C(i,3)*C(i,1)) + (k4(i+1)*C(i,4)*C(i,6));
R_MG = (k3(i+1)*C(i,3)*C(i,1)) - (k4(i+1)*C(i,4)*C(i,6)) - (k5(i+1)*C(i,4)*C(i,1)) + (k6(i+1)*C(i,5)*C(i,6));
R_BD = (k1(i+1)*C(i,2)*C(i,1))-(k2(i+1)*C(i,3)*C(i,6))+(k3(i+1)*C(i,3)*C(i,1))-(k4(i+1)*C(i,4)*C(i,6))+(k5(i+1)*C(i,4)*C(i,1))-(k6(i+1)*C(i,5)*C(i,6));
R_GL = (k5(i+1)*C(i,4)*C(i,1)) - (k6(i+1)*C(i,5)*C(i,6));
R_MetOH = -(k1(i+1)*C(i,2)*C(i,1))+(k2(i+1)*C(i,3)*C(i,6))-(k3(i+1)*C(i,3)*C(i,1))+(k4(i+1)*C(i,4)*C(i,6))-(k5(i+1)*C(i,4)*C(i,1))+(k6(i+1)*C(i,5)*C(i,6));
%We store the molar gain of each component in an R matrix:
R(i,1) = R_MetOH; R(i,2) = R_TG; R(i,3) = R_DG; R(i,4) = R_MG; 
R(i,5) = R_GL; R(i,6) = R_BD;
M(i,:) = R(i,:)*vr; %The change in molar holdup of each component saved in a vector
Msum(i,1) = sum(M(i,:));
end

% IIIb. Evaluate right hand side of DAE-set: M dy/dt = f(y)
f1 = zeros(Ntrays+2,5); 
f2 = zeros(Ntrays+2,1);

% Column
i = 2:Ntrays_react+1;
f2(i,1) = L(i-1,1) - L(i,1) + V(i,1) - V(i-1,1) + Msum(i-1);   

i = Ntrays_react+2:Ntrays+1;
f2(i,1) = L(i-1) - L(i) + V(i,1) - V(i-1,1);

%Correction for the feed:
f2(2,1) = f2(2,1) + F0;


%The molar holdup composition balance for the feed tray:
for j = 1:5;
    for i = 2
        f1(i,j) = ((F0*x_in(1,j)) + (V(i,1)*y(i,j)) + L(i-1,1)*xL(i-1,j) - (V(i-1,1)*y(i-1,j)) - (L(i,1)*xL(i,j)) + M(i-1,j));
    end
end

%The molar holdup composition balance for the two other reactive trays:
for j = 1:5;
    for i = 3:Ntrays_react+1
        f1(i,j) = (V(i,1)*y(i,j)) + (L(i-1,1)*xL(i-1,j) - (V(i-1,1)*y(i-1,j)) - (L(i,1)*xL(i,j)) + M(i-1,j));   
    end
end

%The molar holdup composition balance for the non-reactive trays:
 for j = 1:5;
     for i = Ntrays_react+2:Ntrays+1;
        f1(i,j) = (V(i,1)*y(i,j)) + (L(i-1,1)*xL(i-1,j)) - (V(i-1,1)*y(i-1,j)) - (L(i,1)*xL(i,j));
     end
 end

%Reboiler (assumed to be an equilibrium stage)
i = Ntrays+2;
f2(i,1) = L(Ntrays+1,1) - V(Ntrays+1,1) - B;

j=1:5;
f1(i,j)= (L(Ntrays+1,1)*xL(Ntrays+1,j) - V(Ntrays+1,1)*y(Ntrays+2,j) - B*xL(Ntrays+2,j));

%Total condenser (not an equilibrium stage)
i = 1;
f2(i,1) = V(1,1) - Ref - D;

j = 1:5;
f1(i,j)= V(1,1)*y(i,j) - L(1,1)*xL(i,j) - D*xL(i,j);

%For the algebraic equation for the temperature:
f3 = zeros(Ntrays+2,1);
for i = 1:Ntrays+2
    f3(i,1) = 1 - P(i,1);    %=0, the psat-equations are dependent on temperature
end

f1_1 =(f1 - (xL.*(f2*ones(1,5))))./(HL*ones(1,5));
f1 = reshape(f1_1,(Ntrays+2)*(NC-1),1);

F = [f1; f2; f3];  

